
#include "linux/ppdev.h"
#include "sys/ioctl.h"
#include "fcntl.h"
#include "unistd.h"
#include "time.h"

#include "vflash.h"

static int fd; /* File descriptor for parport device */ 
static int PP_DATA=0x378;
static int PP_STATUS=0x379;
static int PP_CTRL=0x37a;

void Sleep(int millisec)
{
	usleep(millisec*10);
}

void microDelay( unsigned long ms )
{
	int i;
	unsigned char Temp;
    // we need some sleep, that is faster than a Millisecond...
    // http://ldp.paradoxical.co.uk/HOWTO/IO-Port-Programming-4.html
	// 
    // Inputting or outputting any byte from/to port 0x80 should wait 
    // for almost exactly 1 microsecond independent of your processor
    // type and speed. You can do this multiple times to wait a few 
    // microseconds. The port output should have no harmful side effects
    // on any standard machine (and some kernel drivers use it).
	for(i=ms;i>0;i--)
	{
		Temp = _inp(PP_STATUS);
	}
}

int pp_init(int port) 
{ 
	PP_DATA=port;
	PP_STATUS=port+1;
	PP_CTRL=port+2;
	
	fd = open("/dev/parport0", O_RDWR); 
	if (fd==-1) { 
		perror("open"); 
		return 1; 
	} 
 
	if(ioctl(fd, PPCLAIM, NULL)) { 
		perror("PPCLAIM"); 
		close(fd); 
		return 1; 
	} 
 
	return 0; 
} 
 
void _outp(int port, unsigned char data) 
{ 
	int request = PPRSTATUS; 
#if 0
	switch(port) { 
		case PP_DATA:   request = PPWDATA; break; 
		case PP_CTRL:   request = PPWCONTROL; break; 
		default: 
			printf("outp called for unknown port: %x\n", port); 
			return; 
	} 
#endif	
	if(port==PP_DATA) request = PPWDATA;
	else if(port==PP_CTRL) request = PPWCONTROL;
	else printf("outp called for unknown port: %x\n", port); 
	
	if(ioctl(fd, request, &data)) 
	{ 
		perror("Could not write to port"); 
	} 
} 


unsigned char _inp(int port) 
{ 
	int request = PPRSTATUS; 
	unsigned char data; 
#if 0 
	switch(port) { 
		case PP_STATUS: request = PPRSTATUS; break; 
		default: 
			printf("inp called for unknown port: %x\n", port); 
			return 0; 
	} 
#endif	

	if(port==PP_STATUS) request = PPRSTATUS;
	else printf("inp called for unknown port: %x\n", port); 
	
	if(ioctl(fd, request, &data)) { 
		perror("Could not read from port"); 
	} 
	return data; 
} 
 
