
#include "vflash.h"

static int SAFEMODE=1;
static int PARPORT=0x378;
static int XFERDELAY=500; // for microDelay
static bool err_status = 0;

// #############################################################################
// sub_401000
static unsigned char VIPR_Flasher_Send_Cmd( unsigned char Cmd )
{
    unsigned char val, errors;

    errors = 0;
    val = Cmd & 0xF;
    if( ( Cmd & 0x10 ) != 0 ) val |= 0x20;
    _outp( PARPORT, val );

    if( SAFEMODE != 0 )
    {
        if( (_inp(PARPORT+1) & 8) == 0 )
        {
            Sleep(5);
            while( (_inp(PARPORT+1) & 8) == 0 )
            {
                Sleep(5);
                errors++; 
                if(errors >= 5)
                {
                    err_status = 1;
                    return(0);
                }
            }
        }
    }

    val |= 0x10;
    _outp( PARPORT, val );

    if( SAFEMODE != 0 )
    {
        if( (_inp( PARPORT+1 ) & 8) != 0 )
        {
            Sleep(5);
            while( (_inp( PARPORT+1 ) & 8) != 0 )
            {
                Sleep(5);
                errors++; 
                if(errors >= 5)
                {
                    err_status = 1;
                    return(0);
                }
            }
        }
    }

    return(1);
}

// #############################################################################
// sub_4010C0
static void VIPR_Flasher_Init( void )
{
	int i;
    err_status = 0;
    for(i=0;i<0x14;i++)
    {
        VIPR_Flasher_Send_Cmd( 0xFF );
        if( err_status )
            return;
    }
    VIPR_Flasher_Send_Cmd( 0xC );
    VIPR_Flasher_Send_Cmd( 0x12 );
}

// #############################################################################
// sub_401120
static void VIPR_Flasher_Flash_Byte( unsigned char val, unsigned long index )
{
	int i;
    index &= 0x1FFFF;
    VIPR_Flasher_Send_Cmd( 5 );
    if(err_status) 
        return;

    VIPR_Flasher_Send_Cmd( (unsigned char) (((val >> 3) & 0x1C) | (index >> 0xF)) );
    if(err_status) 
        return;

    VIPR_Flasher_Send_Cmd( (unsigned char) (index >> 0xA) );
    if(err_status) 
        return;

    VIPR_Flasher_Send_Cmd( (unsigned char) (index >> 5) );
    if(err_status) 
        return;

    VIPR_Flasher_Send_Cmd( (unsigned char) (index & 0xFF) );
    if(err_status) 
        return;

    for(i=0;i<4;i++)
    {
        VIPR_Flasher_Send_Cmd( val );
    }
}

// #############################################################################
// sub_4011B0
static unsigned char VIPR_Flasher_Read_Response( void )
{
	int i;
    unsigned char val;

    val = 0;
    VIPR_Flasher_Send_Cmd( 0xD );

    for(i=0;i<8;i++)
    {
        val = (((_inp( PARPORT+1 ) >> 3) & 0xFE) << 6) | (val >> 1);
        VIPR_Flasher_Send_Cmd(i);
    }
    return(val);
}

// #############################################################################
// sub_401200
static void VIPR_Flasher_Prepare_Flash( void )
{
	int i;
    unsigned char bResp1, bResp2;

    bResp1 = bResp2 = 0;

    VIPR_Flasher_Send_Cmd( 0x11 );
    for(i=0;i<4;i++)
    {
        VIPR_Flasher_Send_Cmd( 0 );
        if(err_status) break;
    }

    bResp1 = VIPR_Flasher_Read_Response();

    while(bResp1 != bResp2)
    {
        VIPR_Flasher_Send_Cmd( 0x11 );
        for(i=0;i<4;i++)
        {
            VIPR_Flasher_Send_Cmd( 0 );
            if(err_status) break;
        }

        bResp2 = VIPR_Flasher_Read_Response();
    }

    VIPR_Flasher_Read_Response();
    VIPR_Flasher_Read_Response();
    Sleep(1);
    
    return;
}

// #############################################################################
// sub_401300
static bool VIPR_Flasher_IsReady( void )
{
	int i;
    unsigned char bID1, bID2;

    VIPR_Flasher_Send_Cmd( 0x11 );
    for(i=0;i<4;i++)
    {
        VIPR_Flasher_Send_Cmd( 0 );
        if(err_status) break;
    }

    if(err_status)
        return(0);

    for(i=0;i<7;i++)
    {
        VIPR_Flasher_Send_Cmd( 0xE );
    }

    if(err_status)
        return(0);

    bID1 = VIPR_Flasher_Read_Response();
    if(err_status)
        return(0);

    bID2 = VIPR_Flasher_Read_Response();
    if(err_status)
        return(0);

    for(i=0;i<3;i++)
    {
        VIPR_Flasher_Send_Cmd( 0x1B );
    }

    if( bID1 != 0xBF )
    {
        return(0);
    }

    if( bID2 != 0xD5 )
    {
        return(0);
    }

    return(1);
}


// #############################################################################
int Flash_Viper( unsigned char *flash, int fsize )
{
	int i;
    int statcnt;

    // ###############################
    //  First we erase the flash chip
    // ###############################
    printf(" - Erasing the chip... ");
    for(i=0xD; i>0; i--)
    {
        VIPR_Flasher_Send_Cmd( 3 );
    };
    printf("Done!\n");

    // ##################
    //  Prepare flashing
    // ##################
    VIPR_Flasher_Prepare_Flash();

    // #############################
    //  Sleep for 1000 Milliseconds
    // #############################
    Sleep(0x3E8);

    // ########################
    //  Do the actual flashing
    // ########################
    statcnt = 0;
    for(i=0;i<fsize;i++)
    {
        if(flash[i] != 0xFF)
        {
            VIPR_Flasher_Flash_Byte(flash[i],i);
        }

        if((i % 0x250) == 0)
        {
            if(statcnt == 4)
                statcnt = 0;
            if(statcnt == 3)
            {
		    printf(" - Writing data...\r");fflush(stdout);
                statcnt++;
            }
            if(statcnt == 2)
            {
		    printf(" - Writing data.. \r");fflush(stdout);
                statcnt++;
            }
            if(statcnt == 1)
            {
		    printf(" - Writing data.  \r");fflush(stdout);
                statcnt++;
            }
            if(statcnt == 0)
            {
		    printf(" - Writing data   \r");fflush(stdout);
                statcnt++;
            }
        }

        if(err_status)
        {
            printf(" Critical error: Error during communication...\n");
            return(0);
        }

        microDelay(XFERDELAY);
    }
    printf(" - Writing data... Done!\n");

    // #####################
    //  Send the Finish Cmd
    // #####################
    VIPR_Flasher_Send_Cmd( 0 );

    return(1);
}


// #############################################################################
int Read_Viper( unsigned char *flash ,unsigned long len)
{
	unsigned long i;
    int statcnt;

    // #########################
    //  Send the wake up signal
    // #########################
    VIPR_Flasher_Send_Cmd( 0x11 );
    for(i=0;i<4;i++)
    {
        VIPR_Flasher_Send_Cmd( 0 );
        if(err_status) break;
    }

    // #######################
    //  Do the actual reading
    // #######################
    statcnt = 0;
    for(i=0;i<len;i++)
    {    
        flash[i] = VIPR_Flasher_Read_Response();

        if((i % 0x250) == 0)
        {
            if(statcnt == 4)
                statcnt = 0;
            if(statcnt == 3)
            {
		    printf(" - Reading data...\r");fflush(stdout);
                statcnt++;
            }
            if(statcnt == 2)
            {
		    printf(" - Reading data.. \r");fflush(stdout);
                statcnt++;
            }
            if(statcnt == 1)
            {
		    printf(" - Reading data.  \r");fflush(stdout);
                statcnt++;
            }
            if(statcnt == 0)
            {
		    printf(" - Reading data   \r");fflush(stdout);
                statcnt++;
            }
        }

        if(err_status)
        {
            printf(" Critical error: Error during communication...\n");
            return(0);
        }

        microDelay(XFERDELAY);
    }
    printf(" - Reading data... Done!\n");
    
    // #####################
    //  Send the Finish Cmd
    // #####################
    VIPR_Flasher_Send_Cmd( 0 );
    
    return(1);
}

// #############################################################################
int main(int argc, char* argv[])
{
    FILE *fvgc; int fsize; unsigned char *flash; bool readMode;
    int i;unsigned char *flash2; bool writeMode;

    printf("####################################################\n");
    printf(" Viper-GC Flasher Version 0.2 by xor37h and groepaz\n");
    printf("####################################################\n\n");

    if(argc < 2)
    {
        printf("    Usage: vflash.exe <flash.vgc> [-switch]\n\n");

        printf(" Examples: vflash.exe anaconda04.vgc\n");
        printf("           vflash.exe dump.vgc -r\n\n");

        printf(" Switches:\n");
	printf("    -r           read flash to file\n");
	printf("    -v           disable verify when writing\n");
	printf("    -s           disable safe mode\n");
	printf("    -p <addr>    set parport to use (0x378)\n");
	printf("    -d <value>   set delay value (500)\n");

        return(0);
    }


    readMode = 0;
    writeMode = 1;
    if(argc > 2)
    {
	for(i=2;i<argc;i++)
	{
		if((argv[i][0] == '-'))
		{
			switch(argv[i][1])
			{
				case 's':
				case 'S':
					SAFEMODE = 0;
					printf(" safe mode disabled\n");
					break;
				case 'r':
				case 'R':
					readMode = 1;
					break;
				case 'v':
				case 'V':
					writeMode = 0;
					printf(" verify disabled\n");
					break;
				case 'p':
				case 'P':
					i++;
					PARPORT=strtoul(argv[i],NULL,0);
					printf(" using parport address: 0x%04x\n",PARPORT);
					break;
				case 'd':
				case 'D':
					i++;
					XFERDELAY=strtoul(argv[i],NULL,0);
					printf(" using delay value: %d\n",XFERDELAY);
					break;
			}
		}
	}
    }

    pp_init(PARPORT);
    
    // ##############################################
    //  Initialize the Communications with the viper
    // ##############################################
    VIPR_Flasher_Init();
    
    // ########################################
    //  Check for presense of the Viper Module
    // ########################################
    if( VIPR_Flasher_IsReady() )
    {
        printf(" Connected to the Viper GC...\n\n");
    }
     else
    {
        printf(" Connection error: Viper GC not connected...\n");
        return(-2);
    }

    flash = (unsigned char *) malloc(0x20400);

    if(!readMode)
    {
        // ###########################################################
        //  Load the bios and perform some standard validation checks
        // ###########################################################
        fvgc = fopen(argv[1], "rb");
        if (!fvgc) {
            printf(" File error: could not open the file %s\n", argv[1]);
            return(0);
        }

        fseek(fvgc, 0, SEEK_END);
        fsize = ftell(fvgc);
        fseek(fvgc, 0, SEEK_SET);
        fread(flash, 1, fsize, fvgc);

        if ( ( flash[0] != 'V' ) || ( flash[1] != 'I' ) || ( flash[2] != 'P' ) || ( flash[3] != 'R' ) )
        {
            printf(" File error: invalid Viper GC Flash File...\n");
            free(flash);
            return(0);
        }
         else
        {
            printf(" - Loaded BIOS %s named '%s'\n", argv[1], &flash[0x10]);
        }

        // ###################
        //  Flash loaded bios
        // ###################
        Flash_Viper( flash, fsize );
	fclose(fvgc);
    
        // ###################
        //  Verify
        // ###################
	if(writeMode)
	{
		flash2 = (unsigned char *) malloc(fsize);
		VIPR_Flasher_Init();
		Read_Viper( flash2 ,fsize);
		if(memcmp(flash,flash2,fsize)!=0)
		{
			printf(" error: Verify failed :(\n");
		}
		free(flash2);
	}
    }
     else
    {
 
        fvgc = fopen(argv[1],"wb");
        if (!fvgc) {
            printf(" File error: could not open the file %s for writing!\n", argv[1]);
            return(0);
        }

	Read_Viper( flash, 0x20000 );

        fwrite(flash, 1, 0x20000, fvgc);
	fclose(fvgc);
    }

    free(flash);

    return 0;
}
